#pragma once

#include <RingBufCPP.h>
#include <RingBufHelpers.h>

#define ROWS 5
#define COLS 8

#define MKB_BKSP  8
#define MKB_ENTER 13

#define MKB_SPACE ' '
#define MKB_Q     'Q'
#define MKB_W     'W'
#define MKB_E     'E'
#define MKB_R     'R'
#define MKB_T     'T'
#define MKB_Y     'Y'
#define MKB_U     'U'
#define MKB_I     'I'
#define MKB_O     'O'
#define MKB_P     'P'
#define MKB_A     'A'
#define MKB_S     'S'
#define MKB_D     'D'
#define MKB_F     'F'
#define MKB_G     'G'
#define MKB_H     'H'
#define MKB_J     'J'
#define MKB_K     'K'
#define MKB_L     'L'
#define MKB_Z     'Z'
#define MKB_X     'X'
#define MKB_C     'C'
#define MKB_V     'V'
#define MKB_B     'B'
#define MKB_N     'N'
#define MKB_M     'M'
#define MKB_PERIOD '.'

#define MKB_LEFT  0x80
#define MKB_DOWN  0x81
#define MKB_UP    0x82
#define MKB_RIGHT 0x83

#define MKB_SHIFT 0x90
#define MKB_LCTRL 0x91
#define MKB_ALT   0x92
#define MKB_SYM   0x93
#define MKB_FN    0x94
#define MKB_RCTRL 0x95

#define MKB_MOD_SHIFT 0x01
#define MKB_MOD_CTRL  0x02
#define MKB_MOD_ALT   0x04
#define MKB_MOD_SYM   0x08
#define MKB_MOD_FN    0x10

const PROGMEM uint8_t keymap[ROWS][COLS] = {
  {MKB_Q, MKB_W, MKB_R, MKB_Y, MKB_LEFT, MKB_DOWN, MKB_UP, MKB_RIGHT},
  {MKB_A, MKB_D, MKB_E, MKB_T, MKB_U, MKB_I, MKB_O, MKB_P},
  {MKB_SHIFT, MKB_S, MKB_F, MKB_G, MKB_H, MKB_J, MKB_K, MKB_BKSP},
  {MKB_Z, MKB_X, MKB_C, MKB_V, MKB_B, MKB_N, MKB_L, MKB_PERIOD},
  {MKB_LCTRL, MKB_ALT, MKB_SPACE, MKB_SYM, MKB_FN, MKB_RCTRL, MKB_M, MKB_ENTER}
};

typedef struct _KeyEvent {
  uint8_t key;
  bool pressed;
  unsigned long ts;
} KeyEvent;

typedef struct _KeyPress {
  uint8_t key;
  uint8_t modifierState;
} KeyPress;

class MicroKeyboard {
private:
  uint8_t state[ROWS];
  uint8_t previousState[ROWS];
  uint8_t modifierState;
  RingBufCPP<KeyEvent, 16> keyEventBuffer;
  RingBufCPP<KeyPress, 8> keyPressBuffer;
  bool multiModMode;

  bool keyEventInWindow(uint8_t key, bool pressed, unsigned long t_lower_bound);
public:
  MicroKeyboard();

  void scan();
  uint8_t *getState();
  uint8_t getModifierState();
  bool getKeyPress(KeyPress *kp);
};
